"""

    """

import asyncio

from src.mirutil.async_req import get_a_req_async
from src.mirutil.async_req import get_and_render_js_by_requests_html_async as grj


##
url = 'https://codal.ir/Reports/DownloadFile.aspx?id=lCbFkabC4qNOsjayDVl9nQ%3d%3d'

fu = get_a_req_async(url)

r = asyncio.run(fu)

##

r = Res(status = 200 , cnt = 'a')
r

##
url = 'https://codal.ir/Reports/Decision.aspx?LetterSerial=RZQQQaQQQzaaQcLNBAkBMuVzTYXg%3D%3D&rt=0&let=58&ct=0&ft=-1'
fu = grj(url)
r = asyncio.run(fu)

##
r.status_code

##



"""

  """

import re

from persiantools import characters
from persiantools import digits


def convert_digits_to_en(istr) :
    if not isinstance(istr , str) :
        return istr
    os = digits.ar_to_fa(istr)
    os = digits.fa_to_en(os)
    return os

def rm_odd_chars(istr) :
    if not isinstance(istr , str) :
        return istr
    repmap = {
            r"\u202b" : ' ' ,
            r'\u200c' : ' ' ,
            r'\u200d' : '' ,
            r'\u200f' : '' ,
            }
    os = istr
    for ky , vl in repmap.items() :
        os = os.replace(ky , vl)
    return os

def strip_and_rm_successive_spaces(istr) :
    if not isinstance(istr , str) :
        return istr
    os = re.sub('\s+' , ' ' , istr)
    os = os.strip()
    return os

def normalize_fa_str_completely(fa_str: str) -> str :
    if not isinstance(fa_str , str) :
        return fa_str
    os = convert_digits_to_en(fa_str)
    os = characters.ar_to_fa(os)
    os = rm_odd_chars(os)
    os = strip_and_rm_successive_spaces(os)
    return os

def normalize_completley_and_rm_all_whitespaces(fa_str: str) -> str :
    if not isinstance(fa_str , str) :
        return fa_str
    os = normalize_fa_str_completely(fa_str)
    _2rep = {
            '\n'   : None ,
            '\t'   : None ,
            '\r\n' : None ,
            ','    : None ,
            ' '    : None ,
            }
    for k in _2rep.keys() :
        os = os.replace(k , '')
    return os



"""

    """

import json


def read_file(fp , mode = 'r' , encoding = 'utf-8') :
    with open(fp , mode , encoding = encoding) as f :
        return f.read()

def read_txt_file(fp , encoding = 'utf-8') :
    return read_file(fp , encoding = encoding)

def read_json_file(fp) :
    return json.loads(read_txt_file(fp))

def write_to_file(content , fp , mode = 'w' , encoding = 'utf-8') :
    if mode == 'w' :
        txt = content.decode(encoding)
        with open(fp , mode , encoding = encoding) as f :
            f.write(txt)

    elif mode == 'wb' :
        with open(fp , mode) as f :
            f.write(content)

def write_txt_to_file(txt , fp) :
    write_to_file(txt , fp)

async def write_to_file_async(content , fp , mode = 'w' , encoding = 'utf-8') :
    write_to_file(content , fp , mode , encoding)

async def write_txt_to_file_async(txt , fp) :
    write_to_file(txt , fp)



"""

    """

from gitignore_parser import parse_gitignore


def make_tex_code_4_forest_tree_fr_rootdir(root_dir , gitignore_fp) :
    flt = parse_gitignore(gitignore_fp)

    def _build(cur_dir , ignore_filter) :
        res = "\n[" + cur_dir.name
        ls = cur_dir.glob('*')
        dirs = [x for x in ls if x.is_dir()]
        dirs = sorted(dirs)
        for d in dirs :
            if not ignore_filter(d) :
                res += _build(d , ignore_filter)
        return res + "]"

    return _build(root_dir , flt).strip()
